function [ out, org, fixed, dec , flags] = createFrameAnnotation(start, stop, lenX, frmsz, stpsz, deci, fs, varargin)
%CREATEFRAMEANNOTAION Creates annotation vector (@8kHz & @16kHz sampling rate), taking
%into consideration segmentation into frames.
%   Frames are annotated as follows:
%       Speech = 1
%       Non-speech = 0
%       Transient in speech = -1 (defined by _trsp_)
%       Transient in non-speech = -2 (defined by _trnsp_)
%   Input:
%	_start_:    Start-indicies of silence periods @ _fs_.
%	_stop_:     Stop-indicies of silence periods @ _fs_.
%	_lenX_:     Length of whole sound file @ _fs_.
%	_frmsz_:    Frame size @ _fs_/_deci_.
%	_stpsz_:    Step size @ _fs_/_deci_.
%	_deci_:     Decimation factor to decimate signal to _fs_/_deci_.
%	_fs_:       Sampling frequency of signal (full-rate).
%	_varargin_: Start- and stop-indicies of transient periods @ _fs_.
%   Output:
%       _out_:      Frame-based annotation @8kHz.
%       _org_:      Original, sample-based annotation @16kHz. Not
%                   frame-shifted. Not containing transient annotation.
%       _fixed_:    Frame-shifted version of _org_, @16kHz
%       _dec_:      Decimated version of _fixed_, @8kHz
%       _flags_:    Struct containing the flags that have been used in
%                   the annotation.
    
    sp = 1; %%% speech flag
    nsp = 0; %%% non-speech flag
        
    if nargin > 7
        tranStart = varargin{1};
        tranStop = varargin{2};
    else
        tranStart = []; %%% no transients in input, do nothing wrt. transients
        tranStop = [];
    end

    plotDemo = 0;
    minSpeechLength = stpsz/2; %%% determines the lowest number of samples (@ _fs_ Hz) in a speech segment

    numX = fix((lenX-frmsz+stpsz)/stpsz);
    upsampledFrameSize = deci*frmsz;
    upsampledStepSize = deci*stpsz;
    
    org = zeros(1,lenX);
    fixed = 2*ones(1,numX*stpsz-stpsz+frmsz); %%% initialized to twos to easier detect errors
    %%%i = 1;
    if (start(1) ~= 1) %%% if not initial silence (often is)
        org(1:start(1)) = 1; %%% label initial segment as speech
    end
    for i = 2:length(stop)
        if (start(i)-stop(i-1) < minSpeechLength) %%% check if speech segments are unreasonably short (< minSpeechLength)
            continue;
        end
        org(stop(i-1):min(start(i),lenX)) = 1; %%% mark speech segments
    end
    ind = 1:upsampledFrameSize;
    lenOut = ceil(lenX/deci); %%% according to length of decimated signal (ref. decimate() help file).    
    numDecimated = fix((lenOut-frmsz+stpsz)/stpsz);
    out = 2*ones(1, numDecimated); %%% initialized to twos to easier detect errors
    for j = 1:numDecimated
        if (sum(org(ind)) > upsampledFrameSize/2) %%%  %%% i.e. frame is mostly speech
            out(j) = 1; %%% annotate frame as speech
            fixed(ind) = 1; %%% reference vector at full rate (16kHz)
        else
            out(j) = 0;
            fixed(ind) = 0;
        end
        ind = ind + upsampledStepSize;
    end
    
    
    %%%%%%%%%%%%%%
    %%% TRANSIENTS
    trsp = -1; %%% flag to denote transient in speech. Complies with _align_frames_ notation
    trnsp = -2; %%% flag to denote transient in silence/non-speech. Complies with _align_frames_ notation
    
    orgTran = zeros(1,lenX);
    for i = 1:length(tranStart)
        orgTran(tranStart(i):min(tranStop(i),lenX)) = 1; %%% mark transient segments
    end
    ind = 1:upsampledFrameSize;
    for j = 1:numDecimated
        if (sum(orgTran(ind)) > upsampledFrameSize/2) %%%  %%% i.e. frame is mostly transient
            if out(j) == 1
                out(j) = trsp; %%% annotate frame as transient in speech
                fixed(ind) = trsp; %%% reference vector at full rate (16kHz)
            else
                out(j) = trnsp;
                fixed(ind) = trnsp;
            end %%% if not mostly transient, do nothing to _out_ and _fixed_
        end
        ind = ind + upsampledStepSize;
    end
    %%% /TRANSIENTS
    
    dec = downsample(fixed,deci);
    
    flags = {'speech', sp;
            'non-speech', nsp;
            'transient in speech', trsp;
            'transient in non-speech', trnsp};
    
    if plotDemo
        figure();
        subplot(2,1,1)
            plot((1:length(org))/fs, org);
            hold on;
            plot((1:length(fixed))/fs, fixed, 'r:')
            ylim([-0.1 1.1]);
            title('Original, sample-based annotation');
            xlabel('Time [sec]')
            ylabel('Speech = 1 / non-speech = 0')
            legend('Original transcript', 'Frame-shifted transcript')
        subplot(2,1,2)
            %%%frmTicks = (1:length(out));
            plot(out);
            ylim([-0.1 1.1]);
            title('Frame-fixed annotation');
            xlabel('Frame no.');
            ylabel('Speech = 1 / non-speech = 0');

    end
end

